package sun.security.util;

import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.util.regex.Matcher;
import java.util.Locale;
import java.util.regex.Pattern;
import java.math.BigInteger;

public class Debug
{
    private String prefix;
    private static String args;
    private static final char[] hexDigits;
    
    public Debug() {
    }
    
    public static void Help() {
        System.err.println();
        System.err.println("all           turn on all debugging");
        System.err.println("access        print all checkPermission results");
        System.err.println("certpath      PKIX CertPathBuilder and");
        System.err.println("              CertPathValidator debugging");
        System.err.println("combiner      SubjectDomainCombiner debugging");
        System.err.println("gssloginconfig");
        System.err.println("              GSS LoginConfigImpl debugging");
        System.err.println("configfile    JAAS ConfigFile loading");
        System.err.println("configparser  JAAS ConfigFile parsing");
        System.err.println("jar           jar verification");
        System.err.println("logincontext  login context results");
        System.err.println("jca           JCA engine class debugging");
        System.err.println("policy        loading and granting");
        System.err.println("provider      security provider debugging");
        System.err.println("pkcs11        PKCS11 session manager debugging");
        System.err.println("pkcs11keystore");
        System.err.println("              PKCS11 KeyStore debugging");
        System.err.println("sunpkcs11     SunPKCS11 provider debugging");
        System.err.println("scl           permissions SecureClassLoader assigns");
        System.err.println("ts            timestamping");
        System.err.println();
        System.err.println("The following can be used with access:");
        System.err.println();
        System.err.println("stack         include stack trace");
        System.err.println("domain        dump all domains in context");
        System.err.println("failure       before throwing exception, dump stack");
        System.err.println("              and domain that didn't have permission");
        System.err.println();
        System.err.println("The following can be used with stack and domain:");
        System.err.println();
        System.err.println("permission=<classname>");
        System.err.println("              only dump output if specified permission");
        System.err.println("              is being checked");
        System.err.println("codebase=<URL>");
        System.err.println("              only dump output if specified codebase");
        System.err.println("              is being checked");
        System.err.println();
        System.err.println("The following can be used with provider:");
        System.err.println();
        System.err.println("engine=<engines>");
        System.err.println("              only dump output for the specified list");
        System.err.println("              of JCA engines. Supported values:");
        System.err.println("              Cipher, KeyAgreement, KeyGenerator,");
        System.err.println("              KeyPairGenerator, KeyStore, Mac,");
        System.err.println("              MessageDigest, SecureRandom, Signature.");
        System.err.println();
        System.err.println("Note: Separate multiple options with a comma");
        System.exit(0);
    }
    
    public static Debug getInstance(String s) {
        return getInstance(s, s);
    }
    
    public static Debug getInstance(String s, String prefix) {
        if (isOn(s)) {
            Debug debug = new Debug();
            debug.prefix = prefix;
            return debug;
        }
        return null;
    }
    
    public static boolean isOn(String str) {
        return Debug.args != null && (Debug.args.indexOf("all") != -1 || Debug.args.indexOf(str) != -1);
    }
    
    public void println(String str) {
        System.err.println(this.prefix + ": " + str);
    }
    
    public void println() {
        System.err.println(this.prefix + ":");
    }
    
    public static void println(String str, String str2) {
        System.err.println(str + ": " + str2);
    }
    
    public static String toHexString(BigInteger bigInteger) {
        String str = bigInteger.toString(16);
        StringBuffer sb = new StringBuffer(str.length() * 2);
        if (str.startsWith("-")) {
            sb.append("   -");
            str = str.substring(1);
        }
        else {
            sb.append("    ");
        }
        if (str.length() % 2 != 0) {
            str = "0" + str;
        }
        int i = 0;
        while (i < str.length()) {
            sb.append(str.substring(i, i + 2));
            i += 2;
            if (i == str.length()) {
                continue;
            }
            if (i % 64 == 0) {
                sb.append("\n    ");
            }
            else {
                if (i % 8 != 0) {
                    continue;
                }
                sb.append(" ");
            }
        }
        return sb.toString();
    }
    
    private static String marshal(String str) {
        if (str != null) {
            StringBuffer sb = new StringBuffer();
            StringBuffer input = new StringBuffer(str);
            String s = "[Pp][Ee][Rr][Mm][Ii][Ss][Ss][Ii][Oo][Nn]=";
            String replacement = "permission=";
            Matcher matcher = Pattern.compile(s + "[a-zA-Z_$][a-zA-Z0-9_$]*([.][a-zA-Z_$][a-zA-Z0-9_$]*)*").matcher(input);
            StringBuffer sb2 = new StringBuffer();
            while (matcher.find()) {
                sb.append(matcher.group().replaceFirst(s, replacement));
                sb.append("  ");
                matcher.appendReplacement(sb2, "");
            }
            matcher.appendTail(sb2);
            StringBuffer input2 = sb2;
            String s2 = "[Cc][Oo][Dd][Ee][Bb][Aa][Ss][Ee]=";
            String replacement2 = "codebase=";
            Matcher matcher2 = Pattern.compile(s2 + "[^, ;]*").matcher(input2);
            StringBuffer o = new StringBuffer();
            while (matcher2.find()) {
                sb.append(matcher2.group().replaceFirst(s2, replacement2));
                sb.append("  ");
                matcher2.appendReplacement(o, "");
            }
            matcher2.appendTail(o);
            sb.append(o.toString().toLowerCase(Locale.ENGLISH));
            return sb.toString();
        }
        return null;
    }
    
    public static String toString(byte[] array) {
        if (array == null) {
            return "(null)";
        }
        StringBuilder sb = new StringBuilder(array.length * 3);
        for (int i = 0; i < array.length; ++i) {
            int n = array[i] & 0xFF;
            if (i != 0) {
                sb.append(':');
            }
            sb.append(Debug.hexDigits[n >>> 4]);
            sb.append(Debug.hexDigits[n & 0xF]);
        }
        return sb.toString();
    }
    
    static {
        Debug.args = (String)AccessController.doPrivileged(new GetPropertyAction("java.security.debug"));
        String s = (String)AccessController.doPrivileged(new GetPropertyAction("java.security.auth.debug"));
        if (Debug.args == null) {
            Debug.args = s;
        }
        else if (s != null) {
            Debug.args = Debug.args + "," + s;
        }
        if (Debug.args != null) {
            Debug.args = marshal(Debug.args);
            if (Debug.args.equals("help")) {
                Help();
            }
        }
        hexDigits = "0123456789abcdef".toCharArray();
    }
}
